/* Copyright (c) 2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef SRC_SHARED_BOARDS_HELITEST_HELIFMM_FMMSTATUS_H
#define SRC_SHARED_BOARDS_HELITEST_HELIFMM_FMMSTATUS_H

#include <ostream>
#include <string>

enum HeliState : uint8_t
{
    HELISTATE_BOOTED = 0,
    HELISTATE_FAILED,
    HELISTATE_CALIBRATION,
    HELISTATE_READY,
    HELISTATE_FREEFALL,
    HELISTATE_CUTTING,
    HELISTATE_DESCENDING,
    HELISTATE_STOPPING,
    HELISTATE_STOPPED
};

enum FMMFailReason : uint8_t
{
    FMMFAIL_NONE = 0,
    FMMFAIL_LOGGER,
    FMMFAIL_SENSOR
};

struct HeliStatus
{
    uint32_t timestamp;
    uint8_t state      = HELISTATE_BOOTED;
    uint8_t failReason = FMMFAIL_NONE;
    uint32_t time_free_fall, time_start_cut, time_end_cut;

    // void log();

    static std::string header()
    {
        return "timestamp,state,fail_reason,time_free_fall,time_start_cut,time_"
               "end_cut\n";
    }

    void print(std::ostream& os) const
    {
        os << timestamp << "," << (int)state << "," << (int)failReason << ","
           << time_free_fall << "," << time_start_cut << "," << time_end_cut
           << "\n";
    }
};

#endif /* SRC_SHARED_BOARDS_HELITEST_HELIFMM_FMMSTATUS_H */
