/*
 * Copyright (c) 2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <miosix.h>
#include <array>
#include "FMMStatus.h"
#include "Singleton.h"
#include "events/FSM.h"
#include "logger/Logger.h"
#include "HeliTest/ThermalCutter/Cutter.h"

using miosix::FastMutex;
using std::array;

namespace RogallinaBoard
{

class HeliFMM : public FSM<HeliFMM>
{
public:
    HeliFMM(Cutter& cutter);
    ~HeliFMM();

    HeliStatus getStatus();

    void handleEvent(const Event& ev) override;

    void bootedState(const Event& ev);
    void failedState(const Event& ev);
    void calibrationState(const Event& ev);
    void readyState(const Event& ev);
    void freeFallingState(const Event& ev);
    void cuttingState(const Event& ev);
    void descendingState(const Event& ev);
    void stoppingState(const Event& ev);
    void stoppedState(const Event& ev);

    void logStatus()
    {
        status.timestamp = miosix::getTick();
        Logger::instance().log(status);
    }

    uint16_t delayed_ev_id = 0;

    HeliStatus status;
    Logger& log = Logger::instance();

    Cutter& cutter;
};

#define sHeliFMM HeliFMM::getInstance()

}  // namespace RogallinaBoard

