/*
 * Copyright (c) 2019 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <PinObserver.h>
#include <logger/Logger.h>
#include <functional>
#include "HeliTest/Events.h"
#include "HeliTest/config.h"
#include "PinStatus.h"
#include "hwmapping.h"

using std::bind;

namespace RogallinaBoard
{
class PinObserverWrapper
{
public:
    PinObserverWrapper() : observer(PINOBS_POLL_INTERVAL)
    {
        using namespace std::placeholders;

        PinObserver::OnTransitionCallback on_transition =
            bind(&PinObserverWrapper::onTransitionCallback, this, _1, _2);
        PinObserver::OnStateChangeCallback on_state_change =
            bind(&PinObserverWrapper::onStateChangeCallback, this, _1, _2, _3);

        observer.observePin(DETACHMENT_PIN_PORT, DETACHMENT_PIN_NUM,
                            DETACHMENT_PIN_TRANSITION, on_transition,
                            DETACHMENT_PIN_THRESHOLD, on_state_change);

        memset(&status, 0, sizeof(PinStatus));
    }

    ~PinObserverWrapper() {}

    bool start(unsigned int stacksize    = miosix::STACK_DEFAULT_FOR_PTHREAD,
                 miosix::Priority priority = miosix::MAIN_PRIORITY) { return observer.start(stacksize, priority); }

    void stop() { observer.stop(); }

    PinStatus getPinStatus()
    {
        return status;
    }

private:
    void onTransitionCallback(unsigned int p, unsigned char n)
    {
        UNUSED(p);
        UNUSED(n);

        sEventBroker->post({EV_HELIPIN_DISCONNECTED}, TOPIC_FSM);

        status.last_detection = miosix::getTick();
        Logger::instance().log(status);
    }

    void onStateChangeCallback(unsigned int p, unsigned char n,
                               unsigned int state)
    {
        UNUSED(p);
        UNUSED(n);
        status.last_state_change = miosix::getTick();
        ++status.num_state_changes;
        status.state = state;

        Logger::instance().log(status);
    }

    PinObserver observer;
    PinStatus status;
};
}  // namespace RogallinaBoard