/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <PinObserver.h>
#include <miosix.h>
#include <cstdint>
#include "SharedConfig.h"
#include "drivers/pwm/pwm.h"
#include "hwmapping.h"
#include "drivers/HardwareTimer.h"

namespace RogallinaBoard
{
/* CUTTER */
static const PWMChannel CUTTER_PWM_CHANNEL  = PWMChannel::CH2;
static const int CUTTER_PWM_FREQ            = 450;
static const float CUTTER_PWM_DUTY          = 0.07;
static const int CUTTER_PWM_DUTY_RESOLUTION = 1024;

static const PWM::Timer CUTTER_TIM{
    TIM4, &(RCC->APB1ENR), RCC_APB1ENR_TIM4EN,
    TimerUtils::getPrescalerInputFrequency(TimerUtils::InputClock::APB1)};

using CUTTER_ENA    = interfaces::actuators::hbridgel::inh;
using CUTTER_IN     = interfaces::actuators::servol::ctrl;
using CUTTER_CSENSE = miosix::Gpio<GPIOC_BASE, 4>;

/*
 * State machines
 */

// Time from pin detachment to cutter activation in ms
static const unsigned int FREEFALL_TIME_MS = 2.9*1000;
static const unsigned int TIMEOUT_STOP_CUTTING = 15*1000;

static const unsigned int BRIDGE_DISCHARGE_TIME_MS = 30;

// Detachment pin config
static constexpr unsigned int PINOBS_POLL_INTERVAL = 20;

using DETACHMENT_PIN = pin1;
static const miosix::Mode::Mode_ DETACHMENT_PIN_MODE =
    miosix::Mode::INPUT_PULL_UP;

static constexpr unsigned int DETACHMENT_PIN_PORT = pin1_port;
static constexpr unsigned char DETACHMENT_PIN_NUM = pin1_num;

static const PinObserver::Transition DETACHMENT_PIN_TRANSITION =
    PinObserver::Transition::RISING_EDGE;

static constexpr unsigned int DETACHMENT_PIN_THRESHOLD = 5;

}  // namespace RogallinaBoard