/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef SRC_SHARED_BOARDS_HELITEST_ROGALLINA_DESERIALIZE_H
#define SRC_SHARED_BOARDS_HELITEST_ROGALLINA_DESERIALIZE_H

#include <SystemDiagnostics/SystemData.h>
#include <fstream>
#include <iostream>

#include "ButtonStatus.h"
#include "FlightModeManager/FMMStatus.h"
#include "Sensors/GpsData.h"
#include "Sensors/MPU9255Data.h"
#include "logger/Deserializer.h"
#include "logger/LogStats.h"
#include "sensors/BME280/BME280Data.h"
#include "ThermalCutter/CutterData.h"
#include "PinStatus.h"
#include "Sensors/CurrentSenseData.h"
using std::ofstream;

using namespace RogallinaBoard;

template <typename T>
void print(T& t, ostream& os)
{
    t.print(os);
}

void registerTypes(Deserializer& ds)
{
    // LogStats
    ds.registerType<LogStats>(print<LogStats>, LogStats::header());

    // BME280 Data
    ds.registerType<BME280Data>(print<BME280Data>, BME280Data::header());

    // IMU
    ds.registerType<MPU9255Data>(print<MPU9255Data>, MPU9255Data::header());

    // GPS
    ds.registerType<GPSData>(print<GPSData>, GPSData::header());

    // FMM Status
    ds.registerType<HeliStatus>(print<HeliStatus>, HeliStatus::header());

    // Button Status
    ds.registerType<ButtonStatus>(print<ButtonStatus>, ButtonStatus::header());

    // System & thread stats
    ds.registerType<StackData>(print<StackData>, StackData::header());
    ds.registerType<SystemData>(print<SystemData>, SystemData::header());

    // Cutter
    ds.registerType<CutterStatus>(print<CutterStatus>, CutterStatus::header());

    // Current sense
    ds.registerType<CurrentSenseData>(print<CurrentSenseData>, CurrentSenseData::header());

    // Pin Observer
    ds.registerType<PinStatus>(print<PinStatus>, PinStatus::header());
}

#endif
