/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef SRC_SHARED_BOARDS_ROGALLINA_FLIGHTMODEMANAGER_FMMSTATUS_H
#define SRC_SHARED_BOARDS_ROGALLINA_FLIGHTMODEMANAGER_FMMSTATUS_H

#include <ostream>
#include <string>

enum FMMState : uint8_t
{
    FMMSTATE_BOOTED = 0,
    FMMSTATE_FAILED,
    FMMSTATE_TEST_SERVOS,
    FMMSTATE_CALIBRATION,
    FMMSTATE_PROGRAM_SELECT,
    FMMSTATE_PROGRAM_READY,
    FMMSTATE_COUNTDOWN,
    FMMSTATE_WAITING_TRIGGER,
    FMMSTATE_PROGRAM_EXEC,
    FMMSTATE_PROGRAM_ENDED,
    FMMSTATE_STOPPING,
    FMMSTATE_STOPPED
};

enum FMMFailReason : uint8_t
{
    FMMFAIL_NONE = 0,
    FMMFAIL_LOGGER,
    FMMFAIL_SENSOR
};

struct FMMStatus
{
    uint32_t timestamp;
    uint8_t state      = FMMSTATE_BOOTED;
    uint8_t failReason = FMMFAIL_NONE;
    uint8_t programID  = 0;

    uint32_t programStartTime = 0;
    uint32_t programEndedTime = 0;

    // void log();

    static std::string header()
    {
        return "timestamp,state,fail_reason,program_id,program_start_time,"
               "program_end_time\n";
    }

    void print(std::ostream& os) const
    {
        os << timestamp << "," << (int)state << "," << (int)failReason << ","
           << (int)programID << "," << programStartTime << ","
           << programEndedTime << "\n";
    }
};

#endif /* SRC_SHARED_BOARDS_ROGALLINA_FLIGHTMODEMANAGER_FMMSTATUS_H */
