/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef BOARDS_ROGALLINA_FLIGHTMODEMANAGER_ROGALLINAFMM_H
#define BOARDS_ROGALLINA_FLIGHTMODEMANAGER_ROGALLINAFMM_H

#include <miosix.h>
#include <array>
#include "FMMStatus.h"
#include "Singleton.h"
#include "Rogallina/FlightPrograms/ProgramManager.h"
#include "events/FSM.h"
#include "logger/Logger.h"

using miosix::FastMutex;
using std::array;

class ProgramInterface;

static const uint8_t NUM_TESTS = 1;
// Lunghezza countdown prima dell'avvio del test

class RogallinaFMM : public FSM<RogallinaFMM>, public Singleton<RogallinaFMM>
{
    friend class Singleton<RogallinaFMM>;

public:
    FMMStatus getStatus();

protected:
    void handleEvent(const Event& ev) override;

private:
    RogallinaFMM();
    ~RogallinaFMM();

    void bootedState(const Event& ev);

    void failedState(const Event& ev);
    void calibrationState(const Event& ev);
    void testServosState(const Event& ev);
    void programSelectionState(const Event& ev);
    void programReadyState(const Event& ev);
    void countdownState(const Event& ev);
    void waitingTriggerState(const Event& ev);
    void programExecState(const Event& ev);
    void programEndedState(const Event& ev);
    void stoppingState(const Event& ev);
    void stoppedState(const Event& ev);

    void logStatus()
    {
        status.timestamp = miosix::getTick();
        Logger::instance().log(status);
    }

    // Wiggle the servos to show they are working
    void wiggleServos();

    uint16_t delayed_ev_id = 0;

    FMMStatus status;
    ProgramManager* programManager = ProgramManager::getInstance();
    Logger& log                    = Logger::instance();
};

#define sRogallinaFMM RogallinaFMM::getInstance()

#endif  // BOARDS_ROGALLINA_FLIGHTMODEMANAGER_ROGALLINAFMM_H
