/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_PROGRAMSUPERSTALLRECOVERY_H
#define SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_PROGRAMSUPERSTALLRECOVERY_H

#include "../ProgramInterface.h"
#include "Common.h"
#include "Test071018Config.h"
#include "Rogallina/Events.h"
#include "Rogallina/Servo/ServoControl.h"
#include "debug.h"

using miosix::Thread;
class ProgramSuperStallRecovery : public ProgramInterface
{
public:
    ProgramSuperStallRecovery() : servos(ServoControl::getInstance()) {}

    void prepare() override
    {
        TRACE("SuperstallRecovery Preparation\n");
#ifndef ONE_SERVO_AT_A_TIME
        servos->enable();
        servos->start();
        servos->setPositionLeft(0);
        servos->setPositionRight(0);
#else
        setPosLeftHelper(0, SERVO_RESET_TIME);
        setPosRightHelper(0, SERVO_RESET_TIME);
#endif
    }

    void run() override
    {
        TRACE("SuperstallRecovery Running\n");
#ifndef ONE_SERVO_AT_A_TIME
        // Pull all the way
        servos->setPositionLeft(M_PI);
        servos->setPositionRight(M_PI);

        Thread::sleep(SUPERSTALL_RECOVERY_DELAY * 1000);

        servos->setPositionLeft(0);
        servos->setPositionRight(0);
#else
        float step = M_PI / SUPERSTALL_SERVO_STEPS;
        for (int i = 1; i <= SUPERSTALL_SERVO_STEPS; i++)
        {
            setPosLeftHelper(step * i, SUPERSTALL_STEP_INTERVAL);

            setPosRightHelper(step * i, SUPERSTALL_STEP_INTERVAL);
        }

        Thread::sleep(SUPERSTALL_RECOVERY_DELAY * 1000);

        for (int i = SUPERSTALL_SERVO_STEPS - 1; i >= 0; i--)
        {
            setPosLeftHelper(step * i, SUPERSTALL_STEP_INTERVAL);

            setPosRightHelper(step * i, SUPERSTALL_STEP_INTERVAL);
        }
#endif

        TRACE("SuperstallRecovery ended\n");
        sEventBroker->post(Event{EV_PROGRAM_ENDED}, TOPIC_STATUS);
    }

    void cleanUp() override
    {
        TRACE("Superstall CleanUp\n");
#ifndef ONE_SERVO_AT_A_TIME
        servos->setPositionLeft(0);
        servos->setPositionRight(0);

        servos->stop();
        servos->disable();
#else
        setPosLeftHelper(0, SERVO_RESET_TIME);
        setPosRightHelper(0, SERVO_RESET_TIME);
#endif
    }

private:
    void setPosLeftHelper(float pos, long long int sleep)
    {
        servos->enableLeft(true);
        servos->setPositionLeft(pos);
        Thread::sleep(sleep);
        servos->disableLeft();
    }

    void setPosRightHelper(float pos, long long int sleep)
    {
        servos->enableRight(true);
        servos->setPositionRight(pos);
        Thread::sleep(sleep);
        servos->disableRight();
    }

    ServoControl* servos;
};

#endif /* SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_PROGRAMSUPERSTALLRECOVERY_H \
          */
