/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_PROGRAMTEST1_H
#define SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_PROGRAMTEST1_H

#include <cmath>
#include "../ProgramInterface.h"
#include "Common.h"
#include "Test071018Config.h"
#include "Rogallina/Events.h"
#include "Rogallina/Servo/ServoControl.h"
#include "debug.h"

using miosix::Thread;
class ProgramTest1 : public ProgramInterface
{
public:
    ProgramTest1() : servos(ServoControl::getInstance()) {}

    void prepare() override
    {
        TRACE("Test1 Preparation\n");
#ifndef ONE_SERVO_AT_A_TIME
        servos->enable();
        servos->start();
        servos->setPositionLeft(0);
        servos->setPositionRight(0);
#else
        setPosLeftHelper(0, SERVO_RESET_TIME);
        setPosRightHelper(0, SERVO_RESET_TIME);
#endif
    }

    void run() override
    {
        TRACE("Test1 Running\n");
#ifndef ONE_SERVO_AT_A_TIME
        Thread::sleep(TEST1_INITIAL_DELAY_MS);
        servos->setPositionLeft(TEST1_ACTUATION_ANGLE_RAD, TEST1_SPEED);
        servos->setPositionRight(TEST1_ACTUATION_ANGLE_RAD, TEST1_SPEED);
        servos->setPositionLeft(0, TEST1_SPEED);
        servos->setPositionRight(0, TEST1_SPEED);
#else
        Thread::sleep(TEST1_INITIAL_DELAY_MS);

        servos->enableLeft(true);

        servos->setPositionLeft(TEST1_ACTUATION_ANGLE_RAD, TEST1_SPEED);
        Thread::sleep(TEST1_ACTUATION_DELAY);

        servos->disableLeft();
        servos->enableRight(true);

        servos->setPositionRight(TEST1_ACTUATION_ANGLE_RAD, TEST1_SPEED);
        Thread::sleep(TEST1_ACTUATION_DELAY);

        servos->disableRight();
        servos->enableLeft(true);

        servos->setPositionLeft(0, TEST1_SPEED);
        Thread::sleep(TEST1_ACTUATION_DELAY);
        servos->disableLeft();
        servos->enableRight(true);

        servos->setPositionRight(0, TEST1_SPEED);
        Thread::sleep(TEST1_ACTUATION_DELAY);

        servos->disableRight();
#endif

        TRACE("Test1 ended\n");
        sEventBroker->post(Event{EV_PROGRAM_ENDED}, TOPIC_STATUS);
    }

    void cleanUp() override
    {
        TRACE("Test1 CleanUp\n");
#ifndef ONE_SERVO_AT_A_TIME
        servos->setPositionLeft(0);
        servos->setPositionRight(0);

        servos->stop();
        servos->disable();
#else
        setPosLeftHelper(0, SERVO_RESET_TIME);
        setPosRightHelper(0, SERVO_RESET_TIME);
#endif
    }

private:
    void setPosLeftHelper(float pos, long long int sleep)
    {
        servos->enableLeft(true);
        servos->setPositionLeft(pos);
        Thread::sleep(sleep);
        servos->disableLeft();
    }

    void setPosRightHelper(float pos, long long int sleep)
    {
        servos->enableRight(true);
        servos->setPositionRight(pos);
        Thread::sleep(sleep);
        servos->disableRight();
    }

    ServoControl* servos;
};

#endif /* SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_PROGRAMTEST1_H \
          */
