/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_TEST071018CONFIG_H
#define SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_TEST071018CONFIG_H

#include "Rogallina/config.h"
// clang-format off

// SUPERSTALL TEST
static const float 			SUPERSTALL_ANGLE = 35; //degrees
static const int 			SUPERSTALL_RECOVERY_DELAY 	= 5;  // ms
// (Only used if defined ONE_SERVO_AT_A_TIME)
static const int 			SUPERSTALL_SERVO_STEPS		= 10;
static const long long int 	SUPERSTALL_STEP_INTERVAL 	= 200;

// TEST 1
static const float 			TEST1_ACTUATION_ANGLE	= 15;   // Degrees
static const int 			TEST1_INITIAL_DELAY_MS  = 2000; // milliseconds
static const long long int 	TEST1_ACTUATION_DELAY 	= 100;  // milliseconds
#ifdef ONE_SERVO_AT_A_TIME
// Slightly shorter time to account for the sleep at the end to let the servo
// reach  the final position
static const float 			TEST1_ACTUATION_TIME =
							2.0f - TEST1_ACTUATION_DELAY / 1000.0f;    // seconds
#else
static const float 			TEST1_ACTUATION_TIME	= 2.0f;    // seconds
#endif
// (Only used if defined ONE_SERVO_AT_A_TIME)


// Unit conversions & others. Do not modify

static const float TEST1_ACTUATION_ANGLE_RAD
							= TEST1_ACTUATION_ANGLE * M_PI / 180;  // Radians

static const float SUPERSTALL_ANGLE_RAD = SUPERSTALL_ANGLE * M_PI / 180;


static const float TEST1_SPEED =
			TEST1_ACTUATION_ANGLE_RAD / TEST1_ACTUATION_TIME;  // rad / s


static const int TEST1_ACTUATION_TIME_MS = TEST1_ACTUATION_TIME * 1000;  // ms

// clang-format on
#endif /* SRC_SHARED_BOARDS_ROGALLINA_FLIGHTPROGRAMS_TEST071018_TEST071018CONFIG_H \
          */
