/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef SRC_SCREENMANAGER_H
#define SRC_SCREENMANAGER_H

#include <display.h>
#include <misc_inst.h>
#include "FlightModeManager/RogallinaFMM.h"
#include "SensorManager/SensorManager.h"
#include "SystemDiagnostics/SystemDiagnostics.h"
#include "config.h"
#include "display/displayWrapper.h"
#include "logger/Logger.h"

// class RogallinaFMM;
// Type definitions
namespace RogallinaBoard
{
using Sensors::SensorManager;
using Sensors::SensorData;
class ScreenManager
{

public:
    ScreenManager(SensorManager* sm);
    ~ScreenManager();

    void update();

private:
    // Clears the screen
    void clearScreen();

    // Fetch a new status from every component that wants things on the screen
    void fetchData();

    // Update the screen
    void drawScreen();

    // Draw the rogallina FMM status
    void drawRogallinaFMM();
    void drawLoggerStatus();
    void drawSensorSamples();
    void drawSystemStats();

    // Drawing utils
    void write(int x, int y, const char* text, Color txtcolor = black,
               Color bgcolor = white);
    void writeLine(int line, int x, const char* text, Color txtcolor = black,
                   Color bgcolor = white);

    // Data to fetch
    FMMStatus status_fmm;

    LogStats status_logger;
    bool logger_started  = false;
    bool logging_sensors = false;
    string logger_file   = "";

    SensorData sensorData;

    float cpu_usage;
    StackData min_stack;

    // Display handles
    Display& display;
    DrawingContext dc;
    DisplayWrapper dw;

    // Logger
    Logger& logger = Logger::instance();

    SensorManager* sm;
};
}  // namespace RogallinaBoard
#endif /* SRC_SCREENMANAGER_H */
