/* Copyright (c) 2015-2017 Skyward Experimental Rocketry
 * Authors: Luca Erbetta <luca.erbetta@skywarder.eu>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <Common.h>
#include <cstdio>
#include <iostream>
#include <ostream>
#include <string>
#include "logger/Deserializer.h"
#include "logger/Logger.h"

using namespace miosix;
using std::string;
using std::ostream;
using std::cout;
using std::endl;

struct Test1
{
    int a   = 4;
    float c = 5.6f;

    void log(ostream& s)
    {
        s << "1--"
          << "A: " << a << " C : " << c << endl;
    }
};

struct Test2
{
    float f = 545455.5f;
    float d = 5.6f;

    void log(ostream& s)
    {
        s << "2--"
          << "F:" << f << " D:" << d << endl;
    }
};

int main()
{
    Logger& log = Logger::instance();
    int num     = log.start();

    Test1 a;
    Test2 b;
    Test1 c;
    Test1 d;
    Test2 e;

    log.log(a);
    log.log(b);
    log.log(c);
    log.log(d);
    log.log(e);
    log.stop();
    char pd[20];
    sprintf(pd, "log%02d", num);

    Deserializer ss{string(pd), "/sd/"};

    ss.registerType<Test1>([](Test1& t, ostream& os) {
        t.log(os);
        t.log(cout);
    });
    ss.registerType<Test2>([](Test2& t, ostream& os) {
        t.log(os);
        t.log(cout);
    });
    ss.registerType<LogStats>([](LogStats& t, ostream& os) {
        t.print(os);
        t.print(cout);
    });

    ss.deserialize();

    while (true)
    {
        printf("Serial is working!\n");
        Thread::sleep(1000);
    }
}
