/*
 * Copyright (c) 2019 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <drivers/adc/ADC.h>
#include <sensors/Sensor.h>

#include "CurrentSenseData.h"
#include "SharedConfig.h"

/**
 * Abstracts the current sense reading on the adc as if it was an
 * independent sensor.
 */
class CurrentSensor : public Sensor
{
public:
    CurrentSensor() { memset(&current_data, 0, sizeof(CurrentSenseData)); }

    virtual ~CurrentSensor() {}
    
    bool init() override { return true; }

    bool selfTest() override { return true; }

    /**
     * @brief Converts the channels associated with the current sense.
     */
    bool onSimpleUpdate() override
    {
        current_data.timestamp = miosix::getTick();
        current_data.current_value = adc.convertChannel(CS_CHANNEL);
        if(current_data.current_value > current_data.current_max_value)
        {
            current_data.current_max_value = current_data.current_value;
        }
        
        return true;
    }

    /**
     * @brief Returns a pointer to the last conveted current sense value
     */
    CurrentSenseData* getCurrentDataPtr() { return &current_data; }

private:
    /*float adcToCurrent(uint16_t adc_in)
    {
        float v    = (adc_in * 3.3f) / 4096;
        float iout = v / 525;
        return (iout - 0.000030) * 10000;
    }*/

    const ADC_t::Channel CS_CHANNEL =
        static_cast<ADC_t::Channel>(CURRENT_SENSE_ADC_CH);

    ADC_t adc{};
    CurrentSenseData current_data;
};