/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef SRC_SHARED_BOARDS_HELITEST_SENSORMANAGER_SENSORS_GPSDATA_H
#define SRC_SHARED_BOARDS_HELITEST_SENSORMANAGER_SENSORS_GPSDATA_H

#include <cstdint>
#include <ostream>
#include <string>
#include "nmea/minmea.h"  //NMEA parser

struct GPSData
{
    uint32_t timestamp;
    uint8_t buf_count;

    float lat;
    float lon;
    float alt;
    float bearing;
    float speed;

    float filt_lat;
    float filt_long;
    float filt_bearing;
    float filt_speed;

    uint8_t sats;
    uint8_t fix;

    minmea_time time;
    minmea_date date;

    static std::string header()
    {
        return "timestamp,buf_count,lat,lon,alt,bearing,speed,filt_lat,"
               "filt_long,filt_bear,filt_speed,sats,fix,hours,minutes,seconds,"
               "microseconds,day,month,year\n";
    }

    void print(std::ostream& os) const
    {
        os << timestamp << "," << (int)buf_count << "," << lat << "," << lon
           << "," << alt << "," << bearing << "," << speed << "," << filt_lat
           << "," << filt_long << "," << filt_bearing << "," << filt_speed
           << "," << (int)sats << "," << (int)fix << "," << time.hours << ","
           << time.minutes << "," << time.seconds << "," << time.microseconds
           << "," << date.day << "," << date.month << "," << date.year << "\n";
    }
};

#endif /* SRC_SHARED_BOARDS_HELITEST_SENSORMANAGER_SENSORS_GPSDATA_H */
