/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef SRC_SHARED_BOARDS_HELITEST_SYSTEMDIAGNOSTICS_THREADDIAGNOSTICS_H
#define SRC_SHARED_BOARDS_HELITEST_SYSTEMDIAGNOSTICS_THREADDIAGNOSTICS_H

#include <cstdint>
#include <ostream>
#include <string>

struct SystemData
{
    long long timestamp;
    float cpuUsage;
    unsigned int currentFreeHeap;
    unsigned int absoluteFreeHeap;

    static std::string header()
    {
        return "timestamp,cpu_usage,current_heap,absolute_heap\n";
    }

    void print(std::ostream& os)
    {
        os << timestamp << "," << cpuUsage << "," << currentFreeHeap << ","
           << absoluteFreeHeap << "\n";
    }
};

enum ThreadID : uint8_t
{
    THREAD_INVALID = 0,
    THREAD_FIRST   = 1
};

struct StackData
{
    long long int timestamp;
    uint8_t thread_id;
    unsigned int curr_free_stack;
    unsigned int abs_free_stack;

    static std::string header()
    {
        return "timestamp,thread_id,curr_free_stack,abs_free_stack\n";
    }

    void print(std::ostream& os)
    {
        os << timestamp << "," << (int)thread_id << "," << curr_free_stack
           << "," << abs_free_stack << "\n";
    }
};

#endif /* SRC_SHARED_BOARDS_HELITEST_SYSTEMDIAGNOSTICS_THREADDIAGNOSTICS_H */
