/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <chrono>
#include <cstring>
#include <string>
#include <thread>
#include <vector>
#include <mxgui/display.h>
#include <mxgui/entry.h>
#include <mxgui/misc_inst.h>

using namespace mxgui;

class DisplayWrapper
{

private:
    uint16_t h, w, y, step;
    bool firstLine = true;
    Color bg       = black;
    std::vector<std::string> screenvector;

public:
    DisplayWrapper(DrawingContext& dc)
    {
        h = dc.getHeight();
        w = dc.getWidth() *
            1.3;  // FIXME per qualche strano motivo la larghezza non è corretta
        y    = 0;
        step = 10;
    }

    bool setInterspace(int step)
    {
        if (step < h)
        {
            this->step = step;
            return true;
        }
        return false;
    }

    void clearScreen(Color color)
    {
        DrawingContext dc(DisplayManager::instance().getDisplay());
        dc.clear(color);
        bg        = color;
        y         = 0;
        firstLine = true;
    }

    void scroll()
    {
        clearScreen(bg);
        DrawingContext dc(DisplayManager::instance().getDisplay());

        screenvector.erase(screenvector.begin());
        for (std::string& line : screenvector)
        {
            dc.write(Point(0, y), line.c_str());
            y += step;
        }
    }

    void write(uint8_t x_coord, uint8_t y_coord, DrawingContext& dc,
               const char* message)
    {

        int i     = 0;
        int n     = 0;
        y         = 0;
        firstLine = true;

        while (message[i] != '\0')
        {

            if (y + step < h)
            {
                if (!firstLine)
                {
                    y += step;
                }

                for (; message[n] != '\n' && message[n + 1] != '\0' &&
                       n - i < ((w / 7) - 1);
                     n++)
                    ;
                n++;

                char substr[128] = {0};
                memcpy(substr, message + i, n - i);
                substr[n - i + 1] = '\0';
                dc.write(Point(x_coord, y_coord + y), substr);
                screenvector.push_back(std::string(substr));
                i         = n;
                firstLine = false;
            }
        }
        screenvector.clear();
    }

    void writeScrolling(DrawingContext& dc, const char* message)
    {
        // dc.setTextColor(green,black);

        int i = 0;
        int n = 0;

        while (message[i] != '\0')
        {

            if (y + step < h)
            {
                if (!firstLine)
                {
                    y += step;
                }

                for (; message[n] != '\n' && message[n + 1] != '\0' &&
                       n - i < ((w / 7) - 1);
                     n++)
                    ;
                n++;

                char substr[128] = {0};
                memcpy(substr, message + i, n - i);
                substr[n - i + 1] = '\0';
                dc.write(Point(0, y), substr);
                screenvector.push_back(std::string(substr));
                i         = n;
                firstLine = false;
            }
            else
            {
                scroll();
            }
        }
    }
};
