/* Copyright (c) 2015-2018 Skyward Experimental Rocketry
 * Authors: Luca Erbetta
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef SRC_BOARDS_ROGALLINA_HWMAPPING_H
#define SRC_BOARDS_ROGALLINA_HWMAPPING_H

#include <drivers/BusTemplate.h>
#include <miosix.h>
#include <util/software_i2c.h>

using miosix::Gpio;

namespace RogallinaBoard
{

namespace interfaces
{

namespace buttons
{
using user = Gpio<GPIOA_BASE, 0>;
}

namespace spi1
{
using sck  = Gpio<GPIOA_BASE, 5>;
using miso = Gpio<GPIOB_BASE, 4>;
using mosi = Gpio<GPIOA_BASE, 7>;

using bus = BusSPI<1, mosi, miso, sck>;

}  // namespace spi1

namespace i2c
{
using scl = Gpio<GPIOE_BASE, 2>;
using sda = Gpio<GPIOE_BASE, 3>;

using bus = miosix::SoftwareI2C<sda, scl>;
}  // namespace i2c

namespace sensors
{

namespace adc
{
using ch1 = Gpio<GPIOA_BASE, 1>;
using ch2 = Gpio<GPIOA_BASE, 2>;
}  // namespace adc

namespace bme280
{
using cs  = Gpio<GPIOC_BASE, 1>;
using spi = ProtocolSPI<spi1::bus, cs>;
}  // namespace bme280

namespace mpxhz6130a
{
using vout = Gpio<GPIOA_BASE, 2>;
}  // namespace mpxhz6130a

}  // namespace sensors

namespace actuators
{
namespace hbridgel
{
using inh   = Gpio<GPIOC_BASE, 13>;
using in    = Gpio<GPIOC_BASE, 14>;
using csens = Gpio<GPIOF_BASE, 8>;
}  // namespace hbridgel

namespace hbridger
{
using inh   = Gpio<GPIOG_BASE, 2>;
using in    = Gpio<GPIOG_BASE, 3>;
using csens = Gpio<GPIOA_BASE, 1>;
}  // namespace hbridger

namespace servol
{
using ctrl = Gpio<GPIOB_BASE, 7>;
}  // namespace servol

namespace servor
{
using ctrl = Gpio<GPIOB_BASE, 8>;
}  // namespace servor
}  // namespace actuators

}  // namespace interfaces
}  // namespace RogallinaBoard

using green_led = Gpio<GPIOG_BASE, 13>;
using red_led   = Gpio<GPIOG_BASE, 14>;

static constexpr unsigned int pin1_port = GPIOE_BASE;
static constexpr unsigned char pin1_num = 5;

static constexpr unsigned int pin2_port = GPIOE_BASE;
static constexpr unsigned char pin2_num = 6;

static constexpr unsigned int pin3_port = GPIOD_BASE;
static constexpr unsigned char pin3_num = 7;

using pin1 = Gpio<GPIOE_BASE, 5>;
using pin2 = Gpio<GPIOE_BASE, 6>;
using pin3 = Gpio<GPIOD_BASE, 7>;

#endif  // SRC_BOARDS_ROGALLINA_HWMAPPING_H
